@@ -2,13 +2,13 @@ package XML::Fast;
 
 use 5.008008;
 use strict;
-use warnings;
+use warnings;no warnings 'uninitialized';
 use Encode;
 
 use base 'Exporter';
-our @EXPORT_OK = our @EXPORT = qw( xml2hash );
+our @EXPORT_OK = our @EXPORT = qw( xml2hash hash2xml );
 
-our $VERSION = '0.11';
+our $VERSION = '0.16';
 
 use XSLoader;
 XSLoader::load('XML::Fast', $VERSION);
@@ -28,11 +28,26 @@ sub xml2hash($;%) {
 	_xml2hash($xml,\%args);
 }
 
+sub hash2xml($;%) {
+	my $xml = shift;
+	my %args = (
+		order  => 0,        # not impl
+		attr   => '-',      # ok
+		text   => '#text',  # ok
+		join   => '',       # ok
+		trim   => 1,        # ok
+		cdata  => undef,    # ok + fallback -> text
+		comm   => undef,    # ok
+		@_
+	);
+	_hash2xml($xml,\%args);
+}
+
 1;
 __END__
 =head1 NAME
 
-XML::Fast - Simple and very fast XML to hash conversion
+XML::Fast - Simple and very fast XML - hash conversion
 
 =head1 SYNOPSIS
 
@@ -53,8 +68,6 @@ Another similar module is L<XML::Bare>. I've used it for some time, but it have
 
 =over 4
 
-=item * If your XML have node with name 'value', you'll got a segfault
-
 =item * If your XML have node with TextNode, then CDATANode, then again TextNode, you'll got broken value
 
 =item * It doesn't support charsets
@@ -65,25 +78,6 @@ Another similar module is L<XML::Bare>. I've used it for some time, but it have
 
 So, after count of tries to fix L<XML::Bare> I've decided to write parser from scratch.
 
-It is about 40% faster than L<XML::Bare> and about 120% faster, than L<XML::LibXML>
-
-I got this results using the following test on 35kb xml doc:
-
-    cmpthese timethese -10, {
-        libxml  => sub { XML::LibXML->new->parse_string($doc) },
-        xmlfast => sub { XML::Fast::xml2hash($doc) },
-        xmlbare => sub { XML::Bare->new(text => $doc)->parse },
-    };
-
-              Rate  libxml xmlbare xmlfast
-    libxml  1107/s      --    -38%    -56%
-    xmlbare 1782/s     61%      --    -28%
-    xmlfast 2490/s    125%     40%      --
-
-Of course, the results could be defferent for different xml files.
-With non-utf encodings and with many entities it could be slower.
-This test was taken for a sample RSS feed in utf-8 mode with a small count of xml entities.
-
 Here is some features and principles:
 
 =over 4
@@ -98,10 +92,16 @@ Here is some features and principles:
 
 =back
 
+I've removed benchmark results, since they are very different for different xml's.
+Sometimes XML::Bare is faster, sometimes not.
+So, XML::Fast mainly should be considered not C<faster-than-bare>, but C<format-other-than-bare>
+
 =head1 EXPORT
 
 =head2 xml2hash $xml, [ %options ]
 
+=head2 hash2xml $hash, [ %options ]
+
 =head1 OPTIONS
 
 =over 4
@@ -183,6 +183,10 @@ Force nodes with names to be stored as arrays
     # array => ['sub']
     <node><sub/></node>  =>  { node => { sub => [ '' ] } }
 
+=item utf8decode => 1
+
+Force decoding of utf8 sequences, instead of just upgrading them (may be useful for broken xml)
+
 =back
 
 =head1 SEE ALSO
@@ -191,7 +195,7 @@ Force nodes with names to be stored as arrays
 
 =item * L<XML::Bare>
 
-Another fast parser, but have problems
+Another fast parser
 
 =item * L<XML::LibXML>
 
@@ -203,6 +207,14 @@ XML parser, that uses L<XML::LibXML> for parsing and then constructs hash struct
 
 =back
 
+=head1 LIMITATIONS
+
+=over 4
+
+=item * Does not support wide charsets (UTF-16/32) (see L<RT71534|https://rt.cpan.org/Ticket/Display.html?id=71534>)
+
+=back
+
 =head1 TODO
 
 =over 4
